from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Dolyame(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса %s' % type(self).__name__)
        self.__updateCases()

        servicestub.route('/dolyame/create', method='POST', callback=self.create)
        servicestub.route('/dolyame/<orderId>/info', method='GET', callback=self.info)
        servicestub.route('/dolyame/<orderId>/cancel', method='POST', callback=self.cancel)
        servicestub.route('/dolyame/<orderId>/refund', method='POST', callback=self.refund)

    def create(self):
        self.log.info("Создание заказа")
        self.__updateCases()
        methodPath = '/create'

        return self.__getResult(methodPath)

    def info(self, orderId=''):
        self.log.info('Получение информации по заказу %s' % orderId)
        methodPath = '/info'

        return self.__getResult(methodPath)

    def cancel(self, orderId=''):
        self.log.info('Отмена заказа %s' % orderId)
        methodPath = '/cancel'

        return self.__getResult(methodPath)

    def refund(self, orderId=''):
        self.log.info("Возврат по заказу '%s'" % orderId)
        methodPath = '/refund'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            self.log.warn('Путь "%s" не существует' % methodPath)
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if not len(self.cases[methodPath]):
            self.log.warn('В кейсах не осталось значений')
            return None

        time.sleep(self.cases[methodPath][0].get('timeout', 0))

        body = json.dumps(self.cases[methodPath][0].get('response'))
        response = bottle.HTTPResponse(status=self.cases[methodPath][0].get('status'), body=body)
        response.add_header("Content-Type", "application/json")

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodPath][0]:
            return response

        self.cases[methodPath][0]['count'] -= 1

        if self.cases[methodPath][0]['count'] == 0:
            del self.cases[methodPath][0]

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'dolyame.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = Dolyame()
