import json
import os
import time
import xml.etree.ElementTree as ET
from typing import Dict, Any

import bottle
from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Ekp(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}
        self.path = os.path.dirname(os.path.abspath(__file__))

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса Ekp')
        self.__updateCases()

        servicestub.route('/ws/tsp/CardStatus', method='POST', callback=self.handler)

    def handler(self):
        self.__updateCases()
        return self.__getResult("cardStatus")

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        responseFile = str(self.cases[methodPath].get('response')) + '.xml'
        self.log.info("Ответ : %s", responseFile)
        fo = open(os.path.join(self.path, responseFile), "r+")
        response = fo.read()
        time.sleep(self.cases[methodPath].get('timeout', 0))

        response = bottle.HTTPResponse(status=200, body=response)
        response.add_header("Content-Type", "application/xml")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(self.path, 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = Ekp()
