from logging import Logger
import os, json, bottle
from services.basicservice import BasicService
from servicestub import ServiceStub
from singleton import Singleton
import time

class Kjno(BasicService, metaclass=Singleton):
    loginMethod = '/auth/login'
    checkCardMethod = '/kjno-users-info/%s'

    
    def __init__(self):
        super().__init__(os.path.dirname(os.path.abspath(__file__)))

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()

        servicestub.route(Kjno.loginMethod, method='POST', callback=self.login)
        servicestub.route(Kjno.checkCardMethod % '<cardNumber>', method='GET', callback=self.checkCard)
        
    def login(self):
        self.log.info("Авторизация")
        self.__updateCases()
        return self.__getResult(Kjno.loginMethod)
    
    def checkCard(self, cardNumber: str):
        self.log.info(f"Проверка карты: {cardNumber}")
        self.__updateCases()
        return self.__getResult(Kjno.checkCardMethod % cardNumber)
    
    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, f'Путь "{methodPath}" не существует')

        suffix = self.cases[methodPath].get('suffix', '')
        body = self.cases[methodPath]['response' + suffix]
        statusCode = body.get('status', 200)
        time.sleep(self.cases[methodPath].get('timeout', 0))

        if 'status' in body:
            del body['status']
        if 'comment' in body:
            del body['comment']

        response = bottle.HTTPResponse(status=statusCode, body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(self.serviceDir, 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))

instance = Kjno()
