import bottle
import os

from configparser import ConfigParser
from services.basicservice import BasicService
from servicestub import ServiceStub
from singleton import Singleton


class Mfg(BasicService, metaclass=Singleton):
    servicePath: str
    searchCustomerPath = 'searchCustomer'
    loyaltyBalanceInfoPath = 'loyaltyBalanceInfo'
    calculateOrderPath = 'calculateOrder'
    createOrderPath = 'createOrder'
    postOrderPath = 'postOrder'
    cancelOrderPath = 'cancelOrder'

    def __init__(self):
        super().__init__(os.path.dirname(os.path.abspath(__file__)))

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')

        conf = ConfigParser()
        conf.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini'))
        self.servicePath = conf.get('mfg', 'path') if conf.has_option('mfg', 'path') else '/api/ax/xpp/v1'
        self._loadCases('cases.json')

        servicestub.route(os.path.join(self.servicePath, Mfg.searchCustomerPath), method='GET', callback=self.searchCustomer)
        servicestub.route(os.path.join(self.servicePath, Mfg.loyaltyBalanceInfoPath), method='GET',
                callback=self.loyaltyBalanceInfo)
        servicestub.route(os.path.join(self.servicePath, Mfg.calculateOrderPath), method='POST', callback=self.calculateOrder)
        servicestub.route(os.path.join(self.servicePath, Mfg.createOrderPath), method='POST', callback=self.createOrder)
        servicestub.route(os.path.join(self.servicePath, Mfg.postOrderPath), method='POST', callback=self.postOrder)
        servicestub.route(os.path.join(self.servicePath, Mfg.cancelOrderPath), method='POST', callback=self.cancelOrder)

    def searchCustomer(self):
        self.log.info('Поиск клиента по набору параметров')
        return self._createResponse(Mfg.searchCustomerPath)

    def loyaltyBalanceInfo(self):
        self.log.info('Получение баллов для списания')
        return self._createResponse(Mfg.loyaltyBalanceInfoPath)

    def calculateOrder(self):
        self.log.info('Метод предварительного расчёта баллов')
        return self._createResponse(Mfg.calculateOrderPath)

    def createOrder(self):
        self.log.info('Метод создания заказа')
        return self._createResponse(Mfg.createOrderPath)

    def postOrder(self):
        self.log.info('Метод оплаты заказа')
        return self._createResponse(Mfg.postOrderPath)

    def cancelOrder(self):
        self.log.info('Метод для полной отмены заказа')
        return self._createResponse(Mfg.cancelOrderPath)


instance = Mfg()
