from logger import Logger
from services.basicservice import BasicService
from servicestub import ServiceStub
from singleton import Singleton
from typing import Dict, Iterable, Any
import os


class Oplati(BasicService, metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        super().__init__(os.path.dirname(os.path.abspath(__file__)))

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')
        self._loadCases('oplati.json')

        servicestub.route('/tokens', method='GET', callback=self.getToken)
        servicestub.route('/pos/demandPayment', method='POST', callback=self.demandPayment)
        servicestub.route('/pos/consumerStatus', method='GET', callback=self.consumerStatus)
        servicestub.route('/pos/async/payments', method='POST', callback=self.asyncPayments)
        servicestub.route('/pos/payments/<orderNumber>', method='DELETE', callback=self.deleteByOrderNumber)
        servicestub.route('/pos/payments/<orderNumber>/reversals', method='POST', callback=self.reversalByOrderNumber)

    def getToken(self):
        self.log.info("Получение токена")
        methodPath = '/tokens'

        return self._createResponse(methodPath)

    def demandPayment(self):
        self.log.info("Требование оплаты")
        methodPath = '/pos/demandPayment'

        return self._createResponse(methodPath)

    def consumerStatus(self):
        self.log.info("Получение сведений о готовности клиента")
        methodPath = '/pos/consumerStatus'

        return self._createResponse(methodPath)

    def asyncPayments(self):
        self.log.info("Отправить запрос на осуществление платежа (с ожиданием)")
        methodPath = '/pos/async/payments'

        return self._createResponse(methodPath)

    def deleteByOrderNumber(self, orderNumber=''):
        self.log.info("Техническая отмена последней операции: '%s'" % orderNumber)
        methodPath = '/pos/payments/' + orderNumber

        if methodPath not in self.cases:
            methodPath = '/pos/payments/<orderNumber>'

        return self._createResponse(methodPath)

    def reversalByOrderNumber(self, orderNumber=''):
        self.log.info("Отмена операции: '%s'" % orderNumber)
        methodPath = f'/pos/payments/{orderNumber}/reversals'

        if methodPath not in self.cases:
            methodPath = '/pos/payments/<orderNumber>/reversals'

        return self._createResponse(methodPath)


instance = Oplati()
