from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class PriceChecker(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса pricechecker')
        self.__updateCases()

        servicestub.route('/CSrest/rest/checkers/messages/artix', method='GET', callback=self.getPrice)

    def getPrice(self, barcode=''):
        self.__updateCases()
        self.log.info("Получение информации о товаре по штрих-коду")

        params = dict(bottle.request.query.decode())
        barcode = params.get('barcode', '')

        methodPath = '/CSrest/rest/checkers/messages/artix/' + barcode
        if methodPath not in self.cases:
            methodPath = '/CSrest/rest/checkers/messages/artix/<barcode>'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0))
        body = self.cases[methodPath].get('response')

        response = bottle.HTTPResponse(status=self.cases[methodPath].get('status', 200), body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'pricechecker.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = PriceChecker()
