from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class PriceCore(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}
        self.caseNumber = 0

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса PriceCore')
        self.__updateCases()

        servicestub.route('/pricecore/cart/create', method='POST', callback=self.cartCreate)
        servicestub.route('/pricecore/cart/<cartUuid>/add_customer', method='POST', callback=self.addCustomer)
        servicestub.route('/pricecore/cart/<cartUuid>/update', method='POST', callback=self.cartUpdate)
        servicestub.route('/pricecore/cart/<cartUuid>/cart_status', method='POST', callback=self.cartStatus)

    def cartCreate(self):
        self.log.info("Создание корзины")
        return self.__getResult("cartCreate")

    def addCustomer(self, cartUuid=str()):
        self.log.info("Добавление клиента в корзину %s", cartUuid)
        return self.__getResult("addCustomer")

    def cartUpdate(self, cartUuid=str()):
        self.log.info("Обновление корзины %s", cartUuid)
        return self.__getResult("cartUpdate")

    def cartStatus(self, cartUuid=str()):
        self.log.info("Изменение статуса корзины %s", cartUuid)
        return self.__getResult("cartStatus")

    def __getResult(self, method=str()):
        self.__updateCases()

        methodCase = self.cases[method]
        if len(methodCase["caseSeq"]) <= self.caseNumber:
            self.caseNumber = 0
        caseId = methodCase["caseSeq"][self.caseNumber]

        self.log.info('Используется ответ %s', caseId)
        case = methodCase.get(caseId)
        time.sleep(case.get('timeout', 0))

        if case.get('responseFile') is not None and len(case.get('responseFile')) > 0:
            self.log.info('Используется ответ из файла %s', case['responseFile'])
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "cases/" + case['responseFile']), 'rb') as f:
                body = json.load(f)
        else:
            body = json.dumps(case.get('response'))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")
        self.caseNumber = self.caseNumber + 1

        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            for file in ["cases"]:
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = PriceCore()
