from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class ScanBox(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}
        self.caseNumber = 0

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса ScanBox')
        self.__updateCases()

        servicestub.route('/generate', method='POST', callback=self.generate)

    def generate(self):
        self.log.info("Генерация QR-кода")
        self.__updateCases()
        return self.__getResult()

    def __getResult(self):
        if len(self.cases["caseSeq"]) <= self.caseNumber:
            self.caseNumber = 0
        caseId = self.cases["caseSeq"][self.caseNumber]

        self.log.info('Используется ответ %s', caseId)
        case = self.cases.get(caseId)
        time.sleep(case.get('timeout', 0))

        body = json.dumps(case.get('response'))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")
        self.caseNumber = self.caseNumber + 1

        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            for file in ["scanbox"]:
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = ScanBox()
