from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class SmVision(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса smvision')
        self.__updateCases()

        servicestub.route('/smvision/recognize/<partnerId>', method='POST', callback=self.recognize)
        servicestub.route('/smvision/upload', method='POST', callback=self.upload)

    def recognize(self, partnerId:str = ''):
        self.log.info('Распознание товара')
        self.__updateCases()

        methodPath = '/recognize/' + partnerId

        if not methodPath in self.cases:
            methodPath = '/recognize/<partnerId>'

        return self.__getResult(methodPath)

    def upload(self):
        self.log.info('Обновление данных для распознания товара')
        self.__updateCases()

        methodPath = '/upload'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath][0].get('timeout', 0))

        body = json.dumps(self.cases[methodPath][0].get('response'))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'smvision.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = SmVision()
