import os

from services.basicservice import BasicService
from servicestub import ServiceStub
from singleton import Singleton


class WebKassa(BasicService, metaclass=Singleton):
    servicePath: str

    def __init__(self):
        super().__init__(os.path.dirname(os.path.abspath(__file__)))

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')

        self._loadCases('cases.json')

        servicestub.route('/api/Authorize', method='POST', callback=self.getToken)
        servicestub.route('/api/Check', method='POST', callback=self.check)
        servicestub.route('/api/MoneyOperation', method='POST', callback=self.money)
        servicestub.route('/api/ZReport', method='POST', callback=self.zReport)
        servicestub.route('/api/XReport', method='POST', callback=self.xReport)
        servicestub.route('/api/ZReport/ByShiftNumber', method='POST', callback=self.zReportByShiftNumber)
        servicestub.route('/api-history/Reports/ReportByShiftsAndSections', method='POST', callback=self.reportByShiftsAndSections)

    def getToken(self):
        self.log.info("Получение токена")
        methodPath = '/api/Authorize'

        return self._createResponse(methodPath)

    def check(self):
        self.log.info("Регистрация чека")
        methodPath = '/api/Check'

        return self._createResponse(methodPath)

    def money(self):
        self.log.info("Внесение/изъятие наличных")
        methodPath = '/api/MoneyOperation'

        return self._createResponse(methodPath)

    def zReport(self):
        self.log.info("Отчет закрытия смены")
        methodPath = '/api/zReport'

        return self._createResponse(methodPath)

    def xReport(self):
        self.log.info("x-отчет")
        methodPath = '/api/xReport'

        return self._createResponse(methodPath)

    def zReportByShiftNumber(self):
        self.log.info("z-отчет по номеру смены")
        methodPath = '/api/zReportByShiftNumber'

        return self._createResponse(methodPath)

    def reportByShiftsAndSections(self):
        self.log.info("z-отчет по секциям")
        methodPath = '/api-history/Reports/ReportByShiftsAndSections'

        return self._createResponse(methodPath)


instance = WebKassa()
